@@ -426,8 +426,6 @@ static zend_bool php_openssl_matches_san_list(X509 *peer, const char *subject_na
 
 			if (php_openssl_matches_wildcard_name(subject_name, (const char *)cert_name)) {
 				OPENSSL_free(cert_name);
-				sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
-
 				return 1;
 			}
 			OPENSSL_free(cert_name);
@@ -440,8 +438,6 @@ static zend_bool php_openssl_matches_san_list(X509 *peer, const char *subject_na
 					san->d.iPAddress->data[3]
 				);
 				if (strcasecmp(subject_name, (const char*)ipbuffer) == 0) {
-					sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
-
 					return 1;
 				}
 			}
@@ -452,8 +448,6 @@ static zend_bool php_openssl_matches_san_list(X509 *peer, const char *subject_na
 		}
 	}
 
-	sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
-
 	return 0;
 }
 /* }}} */
@@ -487,7 +481,6 @@ static zend_bool php_openssl_matches_common_name(X509 *peer, const char *subject
 static int php_openssl_apply_peer_verification_policy(SSL *ssl, X509 *peer, php_stream *stream) /* {{{ */
 {
 	zval *val = NULL;
-	zval *peer_fingerprint;
 	char *peer_name = NULL;
 	int err,
 		must_verify_peer,
@@ -505,7 +498,6 @@ static int php_openssl_apply_peer_verification_policy(SSL *ssl, X509 *peer, php_
 		: sslsock->is_client;
 
 	must_verify_fingerprint = GET_VER_OPT("peer_fingerprint");
-	peer_fingerprint = val;
 
 	if ((must_verify_peer || must_verify_peer_name || must_verify_fingerprint) && peer == NULL) {
 		php_error_docref(NULL, E_WARNING, "Could not get peer certificate");
@@ -537,8 +529,8 @@ static int php_openssl_apply_peer_verification_policy(SSL *ssl, X509 *peer, php_
 
 	/* If a peer_fingerprint match is required this trumps peer and peer_name verification */
 	if (must_verify_fingerprint) {
-		if (Z_TYPE_P(peer_fingerprint) == IS_STRING || Z_TYPE_P(peer_fingerprint) == IS_ARRAY) {
-			if (!php_openssl_x509_fingerprint_match(peer, peer_fingerprint)) {
+		if (Z_TYPE_P(val) == IS_STRING || Z_TYPE_P(val) == IS_ARRAY) {
+			if (!php_openssl_x509_fingerprint_match(peer, val)) {
 				php_error_docref(NULL, E_WARNING,
 					"peer_fingerprint match failure"
 				);
@@ -827,7 +819,6 @@ static long php_openssl_load_stream_cafile(X509_STORE *cert_store, const char *c
 		buffer_active = 0;
 		if (cert && X509_STORE_add_cert(cert_store, cert)) {
 			++certs_added;
-			X509_free(cert);
 		}
 		goto cert_start;
 	}
@@ -878,7 +869,6 @@ static int php_openssl_enable_peer_verification(SSL_CTX *ctx, php_stream *stream
 
 	if (cafile || capath) {
 		if (!SSL_CTX_load_verify_locations(ctx, cafile, capath)) {
-			ERR_clear_error();
 			if (cafile && !php_openssl_load_stream_cafile(SSL_CTX_get_cert_store(ctx), cafile)) {
 				return FAILURE;
 			}
@@ -1827,7 +1817,7 @@ static int php_openssl_enable_crypto(php_stream *stream,
 {
 	int n;
 	int retry = 1;
-	int cert_captured = 0;
+	int cert_captured;
 	X509 *peer_cert;
 
 	if (cparam->inputs.activate && !sslsock->ssl_active) {
@@ -1872,7 +1862,7 @@ static int php_openssl_enable_crypto(php_stream *stream,
 
 		do {
 			struct timeval cur_time, elapsed_time;
-
+			
 			if (sslsock->is_client) {
 				n = SSL_connect(sslsock->ssl_handle);
 			} else {
@@ -1882,7 +1872,7 @@ static int php_openssl_enable_crypto(php_stream *stream,
 			if (has_timeout) {
 				gettimeofday(&cur_time, NULL);
 				elapsed_time = php_openssl_subtract_timeval(cur_time, start_time);
-
+			
 				if (php_openssl_compare_timeval( elapsed_time, *timeout) > 0) {
 					php_error_docref(NULL, E_WARNING, "SSL: Handshake timed out");
 					return -1;
@@ -2078,7 +2068,7 @@ static size_t php_openssl_sockop_io(int read, php_stream *stream, char *buf, siz
 				if (errno == EAGAIN && err == SSL_ERROR_WANT_WRITE && read == 0) {
 					retry = 1;
 				}
-
+				
 				/* Also, on reads, we may get this condition on an EOF. We should check properly. */
 				if (read) {
 					stream->eof = (retry == 0 && errno != EAGAIN && !SSL_pending(sslsock->ssl_handle));
@@ -2122,7 +2112,7 @@ static size_t php_openssl_sockop_io(int read, php_stream *stream, char *buf, siz
 				}
 			}
 
-			/* Finally, we keep going until we got data, and an SSL_ERROR_NONE, unless we had an error. */
+			/* Finally, we keep going until we got data, and an SSL_ERROR_NONE, unless we had an error. */			
 		} while (retry);
 
 		/* Tell PHP if we read / wrote bytes. */
@@ -2271,11 +2261,10 @@ static inline int php_openssl_tcp_sockop_accept(php_stream *stream, php_openssl_
 	int clisock;
 	zend_bool nodelay = 0;
 	zval *tmpzval = NULL;
-
+	
 	xparam->outputs.client = NULL;
 
-	if (PHP_STREAM_CONTEXT(stream) &&
-		(tmpzval = php_stream_context_get_option(PHP_STREAM_CONTEXT(stream), "socket", "tcp_nodelay")) != NULL &&
+	if ((tmpzval = php_stream_context_get_option(PHP_STREAM_CONTEXT(stream), "socket", "tcp_nodelay")) != NULL &&
 		zend_is_true(tmpzval)) {
 		nodelay = 1;
 	}
@@ -2414,22 +2403,30 @@ static int php_openssl_sockop_set_option(php_stream *stream, int option, int val
 					alive = 0;
 				} else if (php_pollfd_for(sslsock->s.socket, PHP_POLLREADABLE|POLLPRI, &tv) > 0) {
 					if (sslsock->ssl_active) {
-						int n = SSL_peek(sslsock->ssl_handle, &buf, sizeof(buf));
-						if (n <= 0) {
-							int err = SSL_get_error(sslsock->ssl_handle, n);
-							switch (err) {
-								case SSL_ERROR_SYSCALL:
+						int n;
+
+						do {
+							n = SSL_peek(sslsock->ssl_handle, &buf, sizeof(buf));
+							if (n <= 0) {
+								int err = SSL_get_error(sslsock->ssl_handle, n);
+
+								if (err == SSL_ERROR_SYSCALL) {
 									alive = php_socket_errno() == EAGAIN;
 									break;
-								case SSL_ERROR_WANT_READ:
-								case SSL_ERROR_WANT_WRITE:
-									alive = 1;
-									break;
-								default:
-									/* any other problem is a fatal error */
-									alive = 0;
+								}
+
+								if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) {
+									/* re-negotiate */
+									continue;
+								}
+
+								/* any other problem is a fatal error */
+								alive = 0;
 							}
-						}
+							/* either peek succeeded or there was an error; we
+							 * have set the alive flag appropriately */
+							break;
+						} while (1);
 					} else if (0 == recv(sslsock->s.socket, &buf, sizeof(buf), MSG_PEEK) && php_socket_errno() != EAGAIN) {
 						alive = 0;
 					}
