@@ -618,7 +618,7 @@ int phpdbg_compile(void) /* {{{ */
 			data->len += start_line_len;
 			for (i = 1; i <= data->lines; i++) {
 				data->line[i] += start_line_len;
-			}
+			}		
 			zend_hash_update_ptr(&PHPDBG_G(file_sources), PHPDBG_G(ops)->filename, data);
 		}
 
@@ -1158,19 +1158,15 @@ PHPDBG_COMMAND(info) /* {{{ */
 {
 	phpdbg_out("Execution Context Information\n\n");
 	phpdbg_xml("<printinfo %r>");
-#ifdef HAVE_PHPDBG_READLINE
-# ifdef HAVE_LIBREADLINE
-	 phpdbg_writeln("info", "readline=\"yes\"", "Readline   yes");
-# else
-	 phpdbg_writeln("info", "readline=\"no\"", "Readline   no");
-# endif
-# ifdef HAVE_LIBEDIT
-	 phpdbg_writeln("info", "libedit=\"yes\"", "Libedit    yes");
-# else
-	 phpdbg_writeln("info", "libedit=\"no\"", "Libedit    no");
-# endif
+#ifdef HAVE_LIBREADLINE
+	phpdbg_writeln("info", "readline=\"yes\"", "Readline   yes");
 #else
-     phpdbg_writeln("info", "readline=\"unavailable\"", "Readline   unavailable");
+	phpdbg_writeln("info", "readline=\"no\"", "Readline   no");
+#endif
+#ifdef HAVE_LIBEDIT
+	phpdbg_writeln("info", "libedit=\"yes\"", "Libedit    yes");
+#else
+	phpdbg_writeln("info", "libedit=\"no\"", "Libedit    no");
 #endif
 
 	phpdbg_writeln("info", "context=\"%s\"", "Exec       %s", PHPDBG_G(exec) ? PHPDBG_G(exec) : "none");
